//============================================================================
// CFA-10006-12864C Demonstration code
// LOW_LCD.C: Low-level lcd display routines.
// Samsung KS0107+KS0108 (S6B0107+S6B0108) LCD Controller
// Atmel ATMega32 processor @ 16MHz
// WinAVR / AVR GCC + Atmel AVR Studio
// Written by Brent A. Crosby
// Copyright 2005, Crystalfontz America, Inc. http://www.crystalfontz.com
//============================================================================
#include <avr/io.h>
#include <util/delay.h>
#include <avr/iom2561.h>
#include <avr/pgmspace.h>
#include <string.h>
//#include "delay_x.h"
#include "utils.h"
#include "low_lcd.h"
#include "logo_screen.h"
//============================================================================
//This is the display memory.
volatile unsigned char display[8][128];  //1024 bytes

//============================================================================
volatile unsigned char display_status;

//============================================================================
void write_both_lcd_control_registers(unsigned char data)
{

  //We need to wait until both controller's busy bits are clear.
  //Talk to the status (instruction) register.
  CLR_RS;
  //Tell the LCD we are going to be reading it
  SET_RW;
  //Talk to the left controller only
  SET_CS2;
  CLR_CS1;
  //Make port A high-impedance so we can read from the LCD
  LCD_DATA_DDR = LCD_DATA_ALL_INPUTS;
  //Enable the selected controller's read data onto the port
  SET_E;
  _delay_us(6500);
  //Wait for the busy bit to drop
  while(PINA&0x80);
  //Terminate the read
  CLR_E;
  _delay_us(6500);       //stretch LOW E pulse width


  //Talk to the right controller only
  CLR_CS2;
  SET_CS1;
  _delay_us(6500);

  //Enable the selected controller's read data onto the port
  SET_E;
  _delay_us(6500);
  //Wait for the busy bit to drop
  while(PINA&0x80);
  //Terminate the read
  CLR_E;
  _delay_us(6500);    //stretch LOW E pulse width

  //Write the data to the output latches of port A
  LCD_DATA_PORT = data;
  //Tell the LCD that we are writing to it.
  CLR_RW;
  //Talk to both controllers (right controller already active)
  CLR_CS1;
  //Strobe the E pin. We are still aimed at the status/instruction register.
  SET_E;
  //Go back to port A as an output
  LCD_DATA_DDR = LCD_DATA_ALL_OUTPUTS;
  _delay_us(6500);    //stretch LOW E pulse width
  CLR_E;
  _delay_us(6500);    //stretch LOW E pulse width
}

//============================================================================
// PORTC0_LCD_CS1 and PORTD6_LCD_CS2 must be set to select the desired
// controller
//----------------------------------------------------------------------------
void write_lcd_data(unsigned char data)
{

  //We need to wait until both controller's busy bits are clear.
  //Talk to the status (instruction) register.
  CLR_RS;
  //Tell the LCD we are going to be reading it
  SET_RW;
  //Make port A high-impedance
  LCD_DATA_DDR = LCD_DATA_ALL_INPUTS;
  //Enable the selected controller's read data onto the port
  SET_E;
  _delay_us(6500);
  //Wait for the busy bit to drop
  while(PINA&0x80);
  //Terminate the read
  CLR_E;
  _delay_us(6500);      //stretch LOW E pulse width


  //Write the data to the output latches of port A
  LCD_DATA_PORT = data;
  //Tell the LCD that we are writing to it.
  CLR_RW;
  //Aim at the data register.
  SET_RS;
  //Strobe the E pin.
  SET_E;
  //Go back to port A as an output
  LCD_DATA_DDR = LCD_DATA_ALL_OUTPUTS;            //125nS
  _delay_us(6500);     //stretch LOW E pulse width
  //Terminate the write
  CLR_E;
  _delay_us(6500);     //stretch LOW E pulse width
}


//============================================================================
// Takes about 5.08ms (201.7KB/S) reading busy flag
void UpdateLCD(void)
{
     register volatile unsigned char *left_data;
     register volatile unsigned char *right_data;
     register unsigned char page_address;
     register unsigned char column_address;



     //Initialize our source data pointers.
     left_data =& display[0][0];
     right_data =& display[0][64];

     for (page_address = 0; page_address <= 7; page_address++)
     {
          //Set Y address (page, or horizontal stripe of 8 pixels) (controller docs call this "X")
          write_both_lcd_control_registers(0xB8 | page_address);
          //Set X address 0 (right to left) (controller docs call this "Y")
          write_both_lcd_control_registers(0x40);



          for (column_address = 0; column_address <= 63; column_address++)
          {
               //Talk to the left controller only
			   SET_CS2;
			   CLR_CS1;
               write_lcd_data(*left_data++);
               //Talk to the right controller only
			   CLR_CS2;
               SET_CS1;
               write_lcd_data(*right_data++);
          }
          //Skip down to the next line
          left_data += 64;
          right_data += 64;
     }

     // Clear both chip selects
	 SET_CS2;
	 SET_CS1;

}

void ResetLCD(void)
{
     CLR_RESET;
     _delay_us(6500);
     SET_RESET;
}

//============================================================================
void InitializeLCD(void)
{
     /* Take the LCD out of reset (if reset was low)
      */
     SET_RESET;

     /* Never reading status, clear LCD_RW low for writes only
      * Clear LCD_E to low
      */
	 CLR_RW;
	 _delay_us(6500);
     LCD_CTRL_PORT &= ~(1 << LCD_E);

     //Display on
     write_both_lcd_control_registers(0x3F);

     //Set Start Line 0
     write_both_lcd_control_registers(0xC0);

     //Clear the display memeory
     memset(display, 0x00, sizeof(display));
}
//============================================================================
